#!/usr/bin/env python

import struct
import sys
import idaapi
from idc import *

# ida entry point
def accept_file(f, n):
	retval = 0
	if n == 0:
		f.seek(0)
		magic = f.read(4)
		if magic == 'AJ\\b':
			return "Reese Executable"
	return 0

# ida entry point
def load_file(f, neflags, format):
	idaapi.set_processor_type("mipsrl", SETPROC_ALL|SETPROC_FATAL)
	f.seek(0)
	header = f.read(32)
	unk1, unk2, unk3, off_load_sec, off_reg, num_load, num_reg, off_hash, start_addr = struct.unpack("<LLLLLHHLL", header)

	print "off_load  : {}".format(off_load_sec)
	print "num_load  : {}".format(num_load)
	print "off_regs  : {}".format(off_reg)
	print "num_regs  : {}".format(num_reg)
	print "entry     : {}".format(hex(start_addr))
	
	# Only one segment always at same spot
	AddSeg(0x400000, 0x430000, 0, 1, idaapi.saRelPara, idaapi.scPub)
	SetSegmentType(0x400000, idaapi.SEG_CODE)
	RenameSeg(0x400000, "SEG")
	
	for l in range(num_load):
		f.seek(32+20*l)
		head_buff = f.read(20)
		addr, unk1, length, offset, unk2 = struct.unpack("<LLLLL", head_buff)
		
		print "Load Section {}:".format(l)
		print " addr   : {}".format(hex(addr))
		print " offset : {}".format(offset)
		print " length : {}".format(length)
		f.file2base(offset, addr, addr + length, 0)
		
	f.seek(32+20*num_load)
	for r in range(num_reg):
		reg_buff = f.read(8)
		reg, value = struct.unpack("<LL", reg_buff)
		print "Reg[{}] = {}".format(reg, hex(value))
		if reg == 32:
			idaapi.add_entry(value, value, "_start", 1)

		
	return 1
